import pandas as pd
import numpy as np
from collections import Counter
from Bio import SeqIO
from Bio.SeqUtils.ProtParam import ProteinAnalysis
def calculate_base_composition(sequence):
    counts = Counter(sequence)
    total = len(sequence)
    return counts['T'] / total * 100, counts['C'] / total * 100, counts['A'] / total * 100, counts['G'] / total * 100, \
           (counts['A'] + counts['T']) / total * 100, (counts['G'] + counts['C']) / total * 100, total

def calculate_position_base_composition(sequence):
    pos_counts = {1: Counter(), 2: Counter(), 3: Counter()}
    for i, base in enumerate(sequence):
        pos = (i % 3) + 1
        pos_counts[pos][base] += 1

    total_pos = {pos: sum(pos_counts[pos].values()) for pos in pos_counts}
    return (
        pos_counts[1]['T'] / total_pos[1] * 100, pos_counts[1]['C'] / total_pos[1] * 100, pos_counts[1]['A'] / total_pos[1] * 100, pos_counts[1]['G'] / total_pos[1] * 100, total_pos[1],
        pos_counts[2]['T'] / total_pos[2] * 100, pos_counts[2]['C'] / total_pos[2] * 100, pos_counts[2]['A'] / total_pos[2] * 100, pos_counts[2]['G'] / total_pos[2] * 100, total_pos[2],
        pos_counts[3]['T'] / total_pos[3] * 100, pos_counts[3]['C'] / total_pos[3] * 100, pos_counts[3]['A'] / total_pos[3] * 100, pos_counts[3]['G'] / total_pos[3] * 100, total_pos[3]
    )
def features(input_fasta_path, output_excel_path):
    data = []
    for record in SeqIO.parse(input_fasta_path, "fasta"):
        seq_str = str(record.seq)
        t, c, a, g, at_content, gc_content, total = calculate_base_composition(seq_str)
        t1, c1, a1, g1, total1, t2, c2, a2, g2, total2, t3, c3, a3, g3, total3 = calculate_position_base_composition(seq_str)
        features = {
            'Sequence ID': record.id,
            'Length': total,
            'T%': t, 'C%': c, 'A%': a, 'G%': g, 'AT%': at_content, 'GC%': gc_content,
            'T-1': t1, 'C-1': c1, 'A-1': a1, 'G-1': g1, 'Pos #1': total1,
            'T-2': t2, 'C-2': c2, 'A-2': a2, 'G-2': g2, 'Pos #2': total2,
            'T-3': t3, 'C-3': c3, 'A-3': a3, 'G-3': g3, 'Pos #3': total3,
        }
        data.append(features)

    df = pd.DataFrame(data)
    df.to_excel(output_excel_path, index=False)



def calculate_rscu(seq):
    codon_table = {
        'F': ['TTT', 'TTC'], 'L': ['TTA', 'TTG', 'CTT', 'CTC', 'CTA', 'CTG'], 'I': ['ATT', 'ATC', 'ATA'],
        'M': ['ATG'], 'V': ['GTT', 'GTC', 'GTA', 'GTG'], 'S': ['TCT', 'TCC', 'TCA', 'TCG', 'AGT', 'AGC'],
        'P': ['CCT', 'CCC', 'CCA', 'CCG'], 'T': ['ACT', 'ACC', 'ACA', 'ACG'], 'A': ['GCT', 'GCC', 'GCA', 'GCG'],
        'Y': ['TAT', 'TAC'], 'H': ['CAT', 'CAC'], 'Q': ['CAA', 'CAG'], 'N': ['AAT', 'AAC'], 'K': ['AAA', 'AAG'],
        'D': ['GAT', 'GAC'], 'E': ['GAA', 'GAG'], 'C': ['TGT', 'TGC'], 'W': ['TGG'],
        'R': ['CGT', 'CGC', 'CGA', 'CGG', 'AGA', 'AGG'], 'G': ['GGT', 'GGC', 'GGA', 'GGG']
    }

    codon_counts = Counter(seq[i:i + 3] for i in range(0, len(seq) - 2, 3))

    rscu_values = {}
    codon_counts_dict = {}
    for aa, codons in codon_table.items():
        total = sum(codon_counts[codon] for codon in codons)
        n_codons = len(codons)
        for codon in codons:
            observed = codon_counts[codon]
            expected = total / n_codons if n_codons else 0
            rscu = observed / expected if expected else 0
            rscu_values[codon] = rscu
            codon_counts_dict[codon] = observed

    return rscu_values, codon_counts_dict


c2aa = {
    'TGT': 'Cys', 'TGC': 'Cys', 'GAT': 'Asp', 'GAC': 'Asp', 'TCT': 'Ser', 'TCG': 'Ser', 'TCA': 'Ser', 'TCC': 'Ser',
    'AGC': 'Ser', 'AGT': 'Ser',
    'CAA': 'Gln', 'CAG': 'Gln', 'ATG': 'Met', 'AAC': 'Asn', 'AAT': 'Asn', 'CCT': 'Pro', 'CCG': 'Pro', 'CCA': 'Pro',
    'CCC': 'Pro', 'AAG': 'Lys',
    'AAA': 'Lys', 'ACC': 'Thr', 'ACA': 'Thr', 'ACG': 'Thr', 'ACT': 'Thr', 'TTT': 'Phe', 'TTC': 'Phe', 'GCA': 'Ala',
    'GCC': 'Ala', 'GCG': 'Ala',
    'GCT': 'Ala', 'GGT': 'Gly', 'GGG': 'Gly', 'GGA': 'Gly', 'GGC': 'Gly', 'ATC': 'Ile', 'ATA': 'Ile', 'ATT': 'Ile',
    'TTA': 'Leu', 'TTG': 'Leu',
    'CTC': 'Leu', 'CTT': 'Leu', 'CTG': 'Leu', 'CTA': 'Leu', 'CAT': 'His', 'CAC': 'His', 'CGA': 'Arg', 'CGC': 'Arg',
    'CGG': 'Arg', 'CGT': 'Arg',
    'AGG': 'Arg', 'AGA': 'Arg', 'TGG': 'Trp', 'GTA': 'Val', 'GTC': 'Val', 'GTG': 'Val', 'GTT': 'Val', 'GAG': 'Glu',
    'GAA': 'Glu', 'TAT': 'Tyr', 'TAC': 'Tyr'
}


def calculate_third_position_frequencies(seq):
    third_position_counts = Counter()
    total_third_positions = 0

    codons = [seq[i:i + 3] for i in range(0, len(seq) - 2, 3)]
    third_nucleotides = [codon[2] for codon in codons if len(codon) == 3]
    third_position_counts.update(third_nucleotides)
    total_third_positions += len(third_nucleotides)

    T3s = third_position_counts['T'] / total_third_positions if total_third_positions else 0
    C3s = third_position_counts['C'] / total_third_positions if total_third_positions else 0
    A3s = third_position_counts['A'] / total_third_positions if total_third_positions else 0
    G3s = third_position_counts['G'] / total_third_positions if total_third_positions else 0
    GC3s = (third_position_counts['G'] + third_position_counts[
        'C']) / total_third_positions if total_third_positions else 0

    return T3s, C3s, A3s, G3s, GC3s


def calculate_codon_frequency(gene_seq):
    # 计算基因中每个密码子的出现次数
    codon_counts = Counter(gene_seq[i:i + 3] for i in range(0, len(gene_seq), 3))
    total_codons = sum(codon_counts.values())

    # 计算每个密码子的使用频率
    codon_freq = {codon: count / total_codons for codon, count in codon_counts.items()}
    return codon_freq


def calculate_cai(gene_codon_freq, ref_codon_freq):
    # 计算每个密码子的相对适应性值
    cai_values = {codon: gene_freq / ref_codon_freq.get(codon, 1) for codon, gene_freq in gene_codon_freq.items()}

    # 取几何平均值作为 CAI
    cai = np.prod(list(cai_values.values())) ** (1 / len(cai_values))
    return cai


def calculate_cbi(gene_codon_freq):
    # Calculate the standard deviation of codon usage frequency
    codon_freq_values = np.array(list(gene_codon_freq.values()))
    cbi = np.std(codon_freq_values)
    return cbi


def calculate_fop(gene_codon_freq, ref_codon_freq):
    # Calculate the total frequency of optimal codon usage in the gene
    fop = sum(gene_freq for codon, gene_freq in gene_codon_freq.items() if ref_codon_freq.get(codon, 0) == 1)
    return fop


def calculate_nc(gene_codon_freq):
    # Calculate the uniformity of codon usage, i.e., the number of codons
    nc = len(gene_codon_freq)
    return nc
def calculate_L_sym(seq):
    codon_table = {
        'F': ['TTT', 'TTC'], 'L': ['TTA', 'TTG', 'CTT', 'CTC', 'CTA', 'CTG'], 'I': ['ATT', 'ATC', 'ATA'],
        'M': ['ATG'], 'V': ['GTT', 'GTC', 'GTA', 'GTG'], 'S': ['TCT', 'TCC', 'TCA', 'TCG', 'AGT', 'AGC'],
        'P': ['CCT', 'CCC', 'CCA', 'CCG'], 'T': ['ACT', 'ACC', 'ACA', 'ACG'], 'A': ['GCT', 'GCC', 'GCA', 'GCG'],
        'Y': ['TAT', 'TAC'], 'H': ['CAT', 'CAC'], 'Q': ['CAA', 'CAG'], 'N': ['AAT', 'AAC'], 'K': ['AAA', 'AAG'],
        'D': ['GAT', 'GAC'], 'E': ['GAA', 'GAG'], 'C': ['TGT', 'TGC'], 'W': ['TGG'],
        'R': ['CGT', 'CGC', 'CGA', 'CGG', 'AGA', 'AGG'], 'G': ['GGT', 'GGC', 'GGA', 'GGG']
        }
    return sum(seq.count(codon) for codon_list in codon_table.values() for codon in codon_list)

def calculate_L_aa(seq):
    codon_table = {
        'F': ['TTT', 'TTC'], 'L': ['TTA', 'TTG', 'CTT', 'CTC', 'CTA', 'CTG'], 'I': ['ATT', 'ATC', 'ATA'],
        'M': ['ATG'], 'V': ['GTT', 'GTC', 'GTA', 'GTG'], 'S': ['TCT', 'TCC', 'TCA', 'TCG', 'AGT', 'AGC'],
        'P': ['CCT', 'CCC', 'CCA', 'CCG'], 'T': ['ACT', 'ACC', 'ACA', 'ACG'], 'A': ['GCT', 'GCC', 'GCA', 'GCG'],
        'Y': ['TAT', 'TAC'], 'H': ['CAT', 'CAC'], 'Q': ['CAA', 'CAG'], 'N': ['AAT', 'AAC'], 'K': ['AAA', 'AAG'],
        'D': ['GAT', 'GAC'], 'E': ['GAA', 'GAG'], 'C': ['TGT', 'TGC'], 'W': ['TGG'],
        'R': ['CGT', 'CGC', 'CGA', 'CGG', 'AGA', 'AGG'], 'G': ['GGT', 'GGC', 'GGA', 'GGG']
    }
    return len(set(aa for codon_list in codon_table.values() for codon in codon_list for aa, codons in codon_table.items() if codon in codons))

def calculate_gravy(seq):
    protein_analysis = ProteinAnalysis(seq)
    return protein_analysis.gravy()

def calculate_aromo(seq):
    protein_analysis = ProteinAnalysis(seq)
    return protein_analysis.aromaticity()
def rscu(input_fasta_path, output_excel_path):
    seq_records = list(SeqIO.parse(input_fasta_path, 'fasta'))

    rscu_data = []
    summary_data = []

    for rec in seq_records:
        seq = str(rec.seq)
        rscu_values, codon_counts = calculate_rscu(seq)
        T3s, C3s, A3s, G3s, GC3s = calculate_third_position_frequencies(seq)

        for codon, value in rscu_values.items():
            aa = c2aa.get(codon)
            if aa is not None:
                count = codon_counts.get(codon, 0)
                rscu_data.append({
                    'Sequence ID': rec.id,
                    'Codon': codon,
                    'Amino Acid': aa,
                    'RSCU': round(value, 3),
                    'Count': count
                })

        summary_data.append({
            'Sequence ID': rec.id,
            'T3s': T3s,
            'C3s': C3s,
            'A3s': A3s,
            'G3s': G3s,
            'GC3s': GC3s,
            'CAI': calculate_cai(calculate_codon_frequency(seq), reference_codon_freq),
            'CBI': calculate_cbi(calculate_codon_frequency(seq)),
            'Fop': calculate_fop(calculate_codon_frequency(seq), reference_codon_freq),
            'Nc': calculate_nc(calculate_codon_frequency(seq)),
            'L_sym': calculate_L_sym(seq),
            'L_aa': calculate_L_aa(seq),
            'Gravy': calculate_gravy(seq),
            'Aromo': calculate_aromo(seq)
        })

    df_rscu = pd.DataFrame(rscu_data)
    df_summary = pd.DataFrame(summary_data)

    with pd.ExcelWriter(output_excel_path) as writer:
        df_rscu.to_excel(writer, sheet_name='RSCU', index=False)
        df_summary.to_excel(writer, sheet_name='Summary', index=False)


# Example usage
input_fasta_path = 'barcodes_input.fasta'  # Replace with your input FASTA file path
output_excel_path = 'output.xlsx'  # Replace with your output Excel file path

reference_codon_freq = {
    'TTT': 0.5, 'TTC': 0.5, 'TTA': 0.5, 'TTG': 0.5, 'CTT': 0.5, 'CTC': 0.5, 'CTA': 0.5, 'CTG': 0.5,
    'ATT': 0.5, 'ATC': 0.5, 'ATA': 0.5, 'ATG': 1.0, 'GTT': 0.5, 'GTC': 0.5, 'GTA': 0.5, 'GTG': 0.5,
    'TCT': 0.5, 'TCC': 0.5, 'TCA': 0.5, 'TCG': 0.5, 'AGT': 0.5, 'AGC': 0.5, 'CCT': 0.5, 'CCC': 0.5,
    'CCA': 0.5, 'CCG': 0.5, 'ACT': 0.5, 'ACC': 0.5, 'ACA': 0.5, 'ACG': 0.5, 'GCT': 0.5, 'GCC': 0.5,
    'GCA': 0.5, 'GCG': 0.5, 'TAT': 0.5, 'TAC': 0.5, 'TAA': 0.5, 'TAG': 0.5, 'CAT': 0.5, 'CAC': 0.5,
    'CAA': 0.5, 'CAG': 0.5, 'AAT': 0.5, 'AAC': 0.5, 'AAA': 0.5, 'AAG': 0.5, 'GAT': 0.5, 'GAC': 0.5,
    'GAA': 0.5, 'GAG': 0.5, 'TGT': 0.5, 'TGC': 0.5, 'TGA': 0.5, 'TGG': 1.0, 'CGT': 0.5, 'CGC': 0.5,
    'CGA': 0.5, 'CGG': 0.5, 'AGA': 0.5, 'AGG': 0.5, 'GGT': 0.5, 'GGC': 0.5, 'GGA': 0.5, 'GGG': 0.5
}

# # 基本特征
# extract_features(input_fasta_path, output_excel_path)
# #RSCI值计算
# extract_rscu(input_fasta_path, output_excel_path)
#



from docx import Document
from docx.shared import RGBColor, Pt
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT
from Bio import SeqIO
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns

def boxplot(input_excel_path, output_image_path, x_label='', y_label='', title='', font_scale=1.2,
                 figsize=(12, 8)):
    """
    Plots a boxplot from an Excel file.

    Parameters:
    - input_excel_path: str, path to the input Excel file.
    - output_image_path: str, path to save the output image.
    - x_label: str, label for the x-axis.
    - y_label: str, label for the y-axis.
    - title: str, title of the plot.
    - font_scale: float, scale factor for the font size.
    - figsize: tuple, figure size in inches.
    """
    # Read the Excel file
    df = pd.read_excel(input_excel_path)

    # Set up the plot
    plt.figure(figsize=figsize)
    sns.set(font_scale=font_scale)

    # Create the boxplot
    boxplot = sns.boxplot(data=df)

    # Customize the plot with labels and title
    boxplot.set_xlabel(x_label, fontsize=14)
    boxplot.set_ylabel(y_label, fontsize=14)
    boxplot.set_title(title, fontsize=16)

    # Save the plot
    plt.savefig(output_image_path, format=output_image_path.split('.')[-1], dpi=300)
    plt.show()

def onedim(input_file, output_file):
    # Determine the file suffix to choose the processing method
    file_suffix = input_file.split('.')[-1].lower()
    seq_records = []

    # Read the file content
    if file_suffix in ['doc', 'docx']:
        doc = Document(input_file)
        seq_records = [{'id': 'Document Text', 'seq': '\n'.join([p.text for p in doc.paragraphs])}]
    elif file_suffix == 'txt':
        with open(input_file, 'r') as file:
            seq_records = [{'id': 'Text File', 'seq': file.read()}]
    elif file_suffix == 'fasta':
        seq_records = list(SeqIO.parse(input_file, 'fasta'))
    else:
        raise ValueError("Unsupported file format")

    # Replace bases and record the replacement positions and base types
    base_replacements = {
        'A': ('|', 'Showcard Gothic', 14, RGBColor(0x80, 0x76, 0xA3)),  # Size 14 font roughly corresponds to 12pt
        'T': ('|', 'Showcard Gothic', 14, RGBColor(0xFF, 0x00, 0x00)),
        'G': ('|', 'Showcard Gothic', 16, RGBColor(0x56, 0x98, 0xC3)),  # Size 16 font roughly corresponds to 16pt
        'C': ('|', 'Showcard Gothic', 16, RGBColor(0x00, 0xB0, 0x50)),
    }

    # Create a new Word document
    doc = Document()
    # Set the overall style
    style = doc.styles['Normal']
    font = style.font
    font.name = 'Showcard Gothic'

    # Add sequences and their names to the document
    for rec in seq_records:
        # Add sequence ID
        doc.add_paragraph(rec.id, style='Heading 1')

        # Add sequence content
        seq = str(rec.seq)
        for line in seq.split('\n'):
            p = doc.add_paragraph()
            p.alignment = WD_PARAGRAPH_ALIGNMENT.LEFT
            for char in line:
                run = p.add_run(char)
                if char in base_replacements:
                    replacement, font_name, font_size, color = base_replacements[char]
                    run.text = replacement
                    run.font.name = font_name
                    run.font.size = Pt(font_size)
                    run.font.color.rgb = color

    # Save the new Word document
    doc.save(output_file)

def heatmap(input_excel_path, output_image_path, cmap='coolwarm', annot=True, fmt=".3f", font_scale=1.2,
                 figsize=(10, 8)):
    """
    Plots a heatmap from an Excel file.

    Parameters:
    - input_excel_path: str, path to the input Excel file.
    - output_image_path: str, path to save the output image.
    - cmap: str, colormap for the heatmap.
    - annot: bool, whether to annotate the heatmap.
    - fmt: str, string formatting code for annotations.
    - font_scale: float, scale factor for the font size.
    - figsize: tuple, figure size in inches.
    """
    # Read the Excel file
    df = pd.read_excel(input_excel_path, index_col=0)

    # Set up the plot
    plt.figure(figsize=figsize)
    sns.set(font_scale=font_scale)
    heatmap = sns.heatmap(df, cmap=cmap, annot=annot, fmt=fmt, linewidths=.5, cbar_kws={'label': 'Values'})

    # Add title and adjust layout
    heatmap.set_title('Heatmap of Excel Data', fontsize=16)
    plt.xticks(rotation=45, ha='right')
    plt.yticks(rotation=0)
    plt.tight_layout()

    # Save the plot
    plt.savefig(output_image_path, format=output_image_path.split('.')[-1], dpi=300)
    plt.show()


# 1维码可视化
input_file = 'barcodes_input.fasta'  # Replace with the actual file path
output_file = 'output.docx'  # Replace with the desired output file path
one_dim_code(input_file, output_file)

# 热图绘制
input_excel_path = 'heatmap_input.xlsx'  # Replace with the actual input Excel file path
output_image_path = 'heatmap.svg'  # Replace with the desired output image file path, can be 'heatmap.svg', 'heatmap.png', etc.
plot_heatmap(input_excel_path, output_image_path, cmap='viridis', annot=True, fmt=".2f", font_scale=1.5, figsize=(12, 10))

# 箱线图绘制
input_excel_path = 'boxplot_input.xlsx'  # Replace with the actual input Excel file path
output_image_path = 'boxplot.svg'  # Replace with the desired output image file path, can be 'boxplot.svg', 'boxplot.png', etc.
plot_boxplot(input_excel_path, output_image_path, x_label='Samples', y_label='Values', title='Boxplot of Excel Data', font_scale=1.5, figsize=(14, 10))



import os
import pandas as pd
import tkinter as tk
from tkinter import simpledialog

def parse_fasta(lines):
    sequences, current_seq_id = {}, ""
    for line in lines:
        if line.startswith(">"):
            current_seq_id = line.strip()
            sequences[current_seq_id] = ""
        else:
            sequences[current_seq_id] += line.strip()
    return sequences

def modify_and_write_sequences(test_sequences, barcode_sequences, output_folder_path):
    if not os.path.exists(output_folder_path):
        os.makedirs(output_folder_path)

    output_files = []
    for i, (_, barcode_seq) in enumerate(barcode_sequences):
        gap_indices = {index for index, char in enumerate(barcode_seq) if char == "-"}
        output_filename = f"modified_sequences_{i + 1}.txt"
        full_output_path = os.path.join(output_folder_path, output_filename)
        with open(full_output_path, "w") as output_file:
            for _, test_seq in test_sequences:
                modified_seq = ''.join(char for index, char in enumerate(test_seq) if index not in gap_indices)
                output_file.write(modified_seq + "\n")
        output_files.append(full_output_path)
    return output_files

def calculate_recall_rates(barcode_sequence, species_sequences):
    total_correct_nucleotides = 0
    barcode_length = len(barcode_sequence)
    num_species = len(species_sequences)
    recall_rates = {threshold: 0 for threshold in range(90, 100)}

    for species_seq in species_sequences:
        if len(species_seq) < barcode_length:
            continue

        correct_nucleotides = sum(1 for i in range(barcode_length) if species_seq[i] == barcode_sequence[i])
        total_correct_nucleotides += correct_nucleotides

        individual_recall = correct_nucleotides / barcode_length * 100
        for threshold in recall_rates:
            recall_rates[threshold] += 1 if individual_recall > threshold else 0

    average_nucleotide_recall_rate = (total_correct_nucleotides / barcode_length / num_species) * 100
    average_recall_rates = {threshold: (recall / num_species) * 100 for threshold, recall in recall_rates.items()}

    return round(average_nucleotide_recall_rate, 4), average_recall_rates

def recall(input_fasta_path, barcode_file_path, output_excel_path):
    sequences = parse_fasta(open(input_fasta_path, 'r').readlines())

    root = tk.Tk()
    root.withdraw()
    split_index = simpledialog.askinteger("Input", "Please enter the split index for test sequences and barcode sequences:", parent=root, minvalue=1, maxvalue=len(sequences))

    test_sequences = list(sequences.items())[:split_index]
    barcode_sequences = list(sequences.items())[split_index:]

    output_folder_path = 'test_sequences'
    modified_files = modify_and_write_sequences(test_sequences, barcode_sequences, output_folder_path)

    barcodes = read_sequences_from_file(barcode_file_path)
    results = []

    for i, barcode in enumerate(barcodes):
        species_file_name = f"modified_sequences_{i + 1}.txt"
        species_file_path = os.path.join(output_folder_path, species_file_name)

        if not os.path.exists(species_file_path):
            continue

        species_sequences = read_sequences_from_file(species_file_path)
        avg_nuc_recall, avg_recall_rates = calculate_recall_rates(barcode, species_sequences)

        result = {
            'Barcode': barcode,
            'Species File': species_file_name,
            'Average Nucleotide Recall Rate': avg_nuc_recall
        }
        result.update({f'Avg Recall Rate > {threshold}%': rate for threshold, rate in avg_recall_rates.items()})
        results.append(result)

    df = pd.DataFrame(results)
    df.to_excel(output_excel_path, index=False)

def read_sequences_from_file(file_path):
    with open(file_path, 'r') as file:
        return [line.strip() for line in file if line.strip() and '>' not in line]

def parse_fasta(lines):
    sequences, current_seq_id = {}, ""
    for line in lines:
        if line.startswith(">"):
            current_seq_id = line.strip()
            sequences[current_seq_id] = ""
        else:
            sequences[current_seq_id] += line.strip()
    return sequences

def process_and_export_sequences(input_file_path, output_folder_path):
    def modify_and_write_sequences(test_sequences, barcode_sequences, output_folder_path):
        if not os.path.exists(output_folder_path):
            os.makedirs(output_folder_path)

        output_files = []
        for i, (_, barcode_seq) in enumerate(barcode_sequences):
            gap_indices = {index for index, char in enumerate(barcode_seq) if char == "-"}
            output_filename = f"modified_sequences_{i + 1}.txt"
            full_output_path = os.path.join(output_folder_path, output_filename)
            with open(full_output_path, "w") as output_file:
                for _, test_seq in test_sequences:
                    modified_seq = ''.join(char for index, char in enumerate(test_seq) if index not in gap_indices)
                    output_file.write(modified_seq + "\n")
            output_files.append(full_output_path)
        return output_files

    file_extension = input_file_path.split('.')[-1].lower()
    is_fasta_format = file_extension in ['fasta', 'fas', 'txt']

    with open(input_file_path, 'r') as file:
        if is_fasta_format:
            sequences = parse_fasta(file.readlines())
        else:
            raise ValueError("Unsupported file format")

    root = tk.Tk()
    root.withdraw()  # Hide the main window
    split_index = simpledialog.askinteger("Input", "Please enter the split index for test sequences and barcode sequences:", parent=root, minvalue=1, maxvalue=len(sequences))

    test_sequences = list(sequences.items())[:split_index]
    barcode_sequences = list(sequences.items())[split_index:]

    return modify_and_write_sequences(test_sequences, barcode_sequences, output_folder_path)

def read_sequences_from_file(file_path):
    with open(file_path, 'r') as file:
        return [line.strip().replace('-', 'N') for line in file.readlines() if line.strip() and '>' not in line]

def calculate_specificity(barcode_sequence, species_sequences):
    total_nucleotide_diffs = 0
    barcode_length = len(barcode_sequence)
    num_species = len(species_sequences)
    specificity_cases = {n_diffs: 0 for n_diffs in range(1, 11)}

    for species_seq in species_sequences:
        if len(species_seq) < barcode_length:
            continue

        nucleotide_diffs = sum(1 for i in range(barcode_length) if species_seq[i] != barcode_sequence[i])
        total_nucleotide_diffs += nucleotide_diffs

        for n_diffs in specificity_cases:
            specificity_cases[n_diffs] += 100 if nucleotide_diffs >= n_diffs else 0

    average_nucleotide_specificity = (total_nucleotide_diffs / barcode_length / num_species) * 100
    average_specificity_cases = {n_diffs: (specificity / num_species) for n_diffs, specificity in specificity_cases.items()}

    return round(average_nucleotide_specificity, 4), average_specificity_cases

def specificity(input_fasta_path, barcode_file_path, output_excel_path):
    sequences = parse_fasta(open(input_fasta_path, 'r').readlines())

    output_folder_path = 'test_sequences'
    modify_and_write_sequences_output = process_and_export_sequences(input_fasta_path, output_folder_path)

    barcodes = read_sequences_from_file(barcode_file_path)
    results = []

    for i, barcode in enumerate(barcodes):
        species_file_name = f"modified_sequences_{i + 1}.txt"
        species_file_path = os.path.join(output_folder_path, species_file_name)

        if not os.path.exists(species_file_path):
            continue

        species_sequences = read_sequences_from_file(species_file_path)
        avg_nuc_spec, avg_specificity_cases = calculate_specificity(barcode, species_sequences)

        result = {
            'Barcode': barcode,
            'Species File': species_file_name,
            'Average Nucleotide Specificity': avg_nuc_spec
        }
        result.update({f'Avg Specificity (Diff >= {n_diffs})': spec for n_diffs, spec in avg_specificity_cases.items()})
        results.append(result)

    df = pd.DataFrame(results)
    df.to_excel(output_excel_path, index=False)


def primers(barcode_file_path, output_excel_path, primer_length=18):
    """
    Processes a FASTA file to design primers and calculate their properties,
    then saves the data to an Excel file.
    """
    def parse_fasta(file_path):
        sequences = {}
        seq_name = ""
        seq = ""
        with open(file_path, 'r') as file:
            for line in file:
                if line.startswith('>'):
                    if seq_name:
                        sequences[seq_name] = seq
                    seq_name = line.strip()
                    seq = ""
                else:
                    seq += line.strip()
            if seq_name:
                sequences[seq_name] = seq
        return sequences

    def reverse_complement(seq):
        complement = {'A': 'T', 'T': 'A', 'G': 'C', 'C': 'G'}
        return ''.join(complement[base] for base in reversed(seq))

    def calculate_gc_content(primer):
        gc_count = primer.count('G') + primer.count('C')
        return (gc_count / len(primer)) * 100

    def calculate_tm(primer):
        g = primer.count('G')
        c = primer.count('C')
        a = primer.count('A')
        t = primer.count('T')
        return 4 * (g + c) + 2 * (a + t)

    sequences = parse_fasta(barcode_file_path)
    primer_data = []

    for name, seq in sequences.items():
        forward_primer = seq[:primer_length]
        reverse_primer = reverse_complement(seq[-primer_length:])
        primer_data.append({
            'Region': name,
            'Forward Primer': forward_primer,
            'Reverse Primer': reverse_primer,
            'Forward GC%': calculate_gc_content(forward_primer),
            'Reverse GC%': calculate_gc_content(reverse_primer),
            'Forward Tm': calculate_tm(forward_primer),
            'Reverse Tm': calculate_tm(reverse_primer)
        })

    primer_df = pd.DataFrame(primer_data)
    primer_df.to_excel(output_excel_path, index=False)



# input and output files
input_fasta_path = 'input.fasta'  # Replace with the path to your input FASTA file
barcode_file_path = 'barcodes_input.fasta'  # Replace with the path to your barcode file
output_excel_path = 'output.xlsx'  # Replace with the desired output Excel file path

# Calculation of recall
process_files_recall(input_fasta_path, barcode_file_path, output_excel_path)
# Calculation of specificity
process_files_specificity(input_fasta_path, barcode_file_path, output_excel_path)
# primers design
process_files_primers(barcode_file_path, output_excel_path)





from setuptools import setup, find_packages

setup(
    name="flubar",
    version="1.0",
    packages=find_packages(),
    install_requires=[
        'numpy',
        'matplotlib',
        'pandas',
        'biopython',
        'scikit-learn',
        'weblogo'
    ],
    entry_points={
        'console_scripts': [
            'flubarcode = flubar.cli:main'
        ]
    },
    author="DYY",
    description="Barcode sequencing data processing toolkit",
    url='https://github.com/Dyy0426/flubar.git',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
)
